==> sds-stylelint-config-0.5.0.tgz: package/src/utils/validateAttribute.js <==
const chalk = require('chalk');
const { analyzeFuzzy, removeFromSet } = require('./search');
const metadata = require('../metadata/metadata.js');

/**
 * Validate Attribute
 * @param {Object} obj The object from the first Promise that has valuable information about the custom prop
 * @todo Skip scope validation based on stylelint option
 * @returns {Object} Validation object or definition to skip if not found
 */
const validateAttribute = (obj) => {
  if (obj.validate.scope.isGlobal) return { skip: true };

  const results = analyzeFuzzy(obj).filter((item) => item.word);

  // If key has already been stored, we know its valid
  if (obj.keys.attribute) {
    return { valid: true, received: obj.groups[obj.keys.attribute] };
  } else if (obj.fuzzyKeys.size > 0 && results.length > 0) {
    let valid;
    results.map((item) => {
      if (
        (obj.keys.property && obj.keys.state === item.key + 1) ||
        (obj.keys.property && obj.keys.psuedoState === item.key + 1)
      ) {
        obj.keys.attribute = obj.keys.property + 1;
        obj.fuzzyKeys = removeFromSet(obj.fuzzyKeys, obj.keys.attribute);
        valid = {
          valid: false,
          expected: metadata.valid.attribute,
          received: obj.groups[obj.keys.attribute],
        };
      } else if (
        !obj.keys.attribute &&
        results.length === 3 &&
        obj.keys.property &&
        item.key === obj.keys.property + 1
      ) {
        obj.keys.attribute = obj.keys.property + 1;
        obj.fuzzyKeys = removeFromSet(obj.fuzzyKeys, obj.keys.attribute);
        valid = {
          valid: false,
          expected: metadata.valid.attribute,
          received: obj.groups[obj.keys.attribute],
        };
      } else if (!obj.keys.attribute && item.lastKey && item.key === obj.keys.property + 1) {
        valid = { valid: false, received: obj.groups[item.key] };
        obj.customMessage.push(
          `We received "${chalk.bold(
            obj.groups[item.key],
          )}" and have low confidence in determining if this is an attribute, state, or psuedoState. Possible values are ${chalk.bold(
            metadata.valid.attribute,
          )},${chalk.bold(metadata.valid.state)},${chalk.bold(metadata.valid.psuedoState)}.`,
        );
      } else if (
        !obj.keys.attribute &&
        !item.lastKey &&
        item.key === obj.keys.property + 1 &&
        results.length <= 2
      ) {
        valid = { valid: false, received: obj.groups[item.key] };
        obj.customMessage.push(
          `We received "${chalk.bold(
            obj.groups[item.key],
          )}" and have medium confidence that this is an attribute but might also be a state, or psuedoState. Possible values are ${chalk.bold(
            metadata.valid.attribute,
          )},${chalk.bold(metadata.valid.state)},${chalk.bold(metadata.valid.psuedoState)}.`,
        );
      } else if (obj.keys.property !== obj.keys.category + 1) {
        valid = { valid: false, received: obj.groups[item.key] };
        obj.customMessage.push(
          `We received "${chalk.bold(
            obj.groups[item.key],
          )}" but we do not allow words in between "${chalk.bold(
            obj.groups[obj.keys.category],
          )}", a category, and "${chalk.bold(obj.groups[obj.keys.property])}", a property.`,
        );
      }
    });
    return valid;
  } else {
    return { skip: true };
  }
};

module.exports = validateAttribute;

==> sds-stylelint-config-1.0.1.tgz: package/src/utils/validateAttribute.js <==
const chalk = require('chalk');
const { analyzeFuzzy, removeFromSet } = require('./search');
const metadata = require('../metadata/metadata.js');

/**
 * Validate Attribute
 * @param {Object} obj The object from the first Promise that has valuable information about the custom prop
 * @todo Skip scope validation based on stylelint option
 * @returns {Object} Validation object or definition to skip if not found
 */
const validateAttribute = (obj) => {
  if (obj.validate.scope.isGlobal || obj.validate.scope.isShared) return { skip: true };

  const results = analyzeFuzzy(obj).filter((item) => item.word);

  // If key has already been stored, we know its valid
  if (obj.keys.attribute) {
    return { valid: true, received: obj.groups[obj.keys.attribute] };
  } else if (obj.fuzzyKeys.size > 0 && results.length > 0) {
    let valid;
    results.map((item) => {
      if (
        (obj.keys.property && obj.keys.state === item.key + 1) ||
        (obj.keys.property && obj.keys.psuedoState === item.key + 1)
      ) {
        obj.keys.attribute = obj.keys.property + 1;
        obj.fuzzyKeys = removeFromSet(obj.fuzzyKeys, obj.keys.attribute);
        valid = {
          valid: false,
          expected: metadata.component.valid.attribute,
          received: obj.groups[obj.keys.attribute],
        };
      } else if (
        !obj.keys.attribute &&
        results.length === 3 &&
        obj.keys.property &&
        item.key === obj.keys.property + 1
      ) {
        obj.keys.attribute = obj.keys.property + 1;
        obj.fuzzyKeys = removeFromSet(obj.fuzzyKeys, obj.keys.attribute);
        valid = {
          valid: false,
          expected: metadata.component.valid.attribute,
          received: obj.groups[obj.keys.attribute],
        };
      } else if (!obj.keys.attribute && item.lastKey && item.key === obj.keys.property + 1) {
        valid = { valid: false, received: obj.groups[item.key] };
        obj.customMessage.push(
          `We received "${chalk.bold(
            obj.groups[item.key],
          )}" and have low confidence in determining if this is an attribute, state, or psuedoState. Possible values are ${chalk.bold(
            metadata.component.valid.attribute,
          )},${chalk.bold(metadata.component.valid.state)},${chalk.bold(
            metadata.component.valid.psuedoState,
          )}.`,
        );
      } else if (
        !obj.keys.attribute &&
        !item.lastKey &&
        item.key === obj.keys.property + 1 &&
        results.length <= 2
      ) {
        valid = { valid: false, received: obj.groups[item.key] };
        obj.customMessage.push(
          `We received "${chalk.bold(
            obj.groups[item.key],
          )}" and have medium confidence that this is an attribute but might also be a state, or psuedoState. Possible values are ${chalk.bold(
            metadata.component.valid.attribute,
          )},${chalk.bold(metadata.component.valid.state)},${chalk.bold(
            metadata.component.valid.psuedoState,
          )}.`,
        );
      } else if (obj.keys.property !== obj.keys.category + 1) {
        valid = { valid: false, received: obj.groups[item.key] };
        obj.customMessage.push(
          `We received "${chalk.bold(
            obj.groups[item.key],
          )}" but we do not allow words in between "${chalk.bold(
            obj.groups[obj.keys.category],
          )}", a category, and "${chalk.bold(obj.groups[obj.keys.property])}", a property.`,
        );
      }
    });
    return valid;
  } else {
    return { skip: true };
  }
};

module.exports = validateAttribute;
